package edu.nju.algorithm.progress;

import edu.nju.entities.Bug;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import java.util.TreeMap;

/**
 * @Author JiaWei Xu
 * @Date 2021-01-06 16:02
 * @Email xjwhhh233@outlook.com
 */
public class TaskClosePrediction {
    public Double determineTaskProgressStatus(List<BugForProgress> bugForProgressList) {
        Random rand = new Random();

        ArrayList<Integer[]> captureResults = new ArrayList<Integer[]>();
        TreeMap<Integer, ArrayList<BugForProgress>> captureProcess = new TreeMap<Integer, ArrayList<BugForProgress>>();

        int captureSize = ConstantsForProgress.captureSize;
        int equalTimeThres = ConstantsForProgress.equalTimeThres;

        if (bugForProgressList.size() / captureSize < 5) {
            return bugForProgressList.size() * 2.0;
        }

        M0CRCAlgorithm M0Algorithm = new M0CRCAlgorithm();
        int captureTime = -1;
        for (int i = 0; i < bugForProgressList.size(); ) {
            captureTime++;
            int beginReport = captureTime * captureSize;
            int endReport = beginReport + captureSize;

            if (endReport >= bugForProgressList.size()) {
                break;
            }

            ArrayList<BugForProgress> curCapture = new ArrayList<BugForProgress>();
            for (int j = beginReport; j < endReport && j < bugForProgressList.size(); j++) {
                curCapture.add(bugForProgressList.get(j));
            }
            captureProcess.put(captureTime, curCapture);

            Integer[] results = M0Algorithm.obtainRecaptureResults(captureProcess);    //estBugs, estBugs, estBugs, alreadySubBugs
            captureResults.add(results);
        }

        Boolean isTerminate = this.whetherCanTerminate(captureResults, equalTimeThres);
        Integer[] lastResults = captureResults.get(captureResults.size() - 1);
        Double bugRatio = 1.0 * lastResults[3] / lastResults[0];
        //System.out.println ( "bug ratio : " + bugRatio + " whether can terminate: " + isTerminate );

        bugRatio = bugRatio * 100;
        if (!isTerminate) {
            if (bugRatio > 98) {
                bugRatio = rand.nextInt(4) + 94.0;
            }
        }

        return bugRatio;
    }

    private Boolean whetherCanTerminate(ArrayList<Integer[]> captureResults, Integer equalTimeThres) {
        if (captureResults.size() < equalTimeThres) {
            return false;
        }

        int count = 0;
        for (int i = captureResults.size() - 1; i > 0; i--) {
            Integer[] curResults = captureResults.get(i);
            Integer[] priorResults = captureResults.get(i - 1);

            if (this.isEqual(priorResults, curResults)) {
                count++;
            } else {
                break;
            }

            if (count >= equalTimeThres) {
                return true;
            }
        }
        return false;
    }

    private Boolean isEqual(Integer[] priorResults, Integer[] curResults) {
        if (!curResults[0].equals(curResults[3])) {
            return false;
        }
        if (curResults[3] == 0) {
            return false;
        }
        return true;
    }
}
